@@ -4,19 +4,20 @@
 
 #include <clutter/x11/clutter-x11.h>
 
-#include "screen.h"
-#include "errors.h"
-#include "window.h"
+#include <meta/screen.h>
+#include <meta/errors.h>
+#include <meta/window.h>
 #include "compositor-private.h"
-#include "compositor-mutter.h"
+#include <meta/compositor-mutter.h>
 #include "xprops.h"
-#include "prefs.h"
-#include "meta-shadow-factory.h"
+#include <meta/prefs.h>
+#include <meta/main.h>
+#include <meta/meta-shadow-factory.h>
 #include "meta-window-actor-private.h"
 #include "meta-window-group.h"
 #include "meta-background-actor.h"
-#include "../core/window-private.h" /* to check window->hidden */
-#include "../core/display-private.h" /* for meta_display_lookup_x_window() */
+#include "window-private.h" /* to check window->hidden */
+#include "display-private.h" /* for meta_display_lookup_x_window() */
 #include <X11/extensions/shape.h>
 #include <X11/extensions/Xcomposite.h>
 
@@ -474,20 +475,43 @@ meta_compositor_manage_screen (MetaCompositor *compositor,
   gint            width, height;
   XWindowAttributes attr;
   long            event_mask;
+  guint           n_retries;
+  guint           max_retries;
 
   /* Check if the screen is already managed */
   if (meta_screen_get_compositor_data (screen))
     return;
 
-  meta_error_trap_push_with_return (display);
-  XCompositeRedirectSubwindows (xdisplay, xroot, CompositeRedirectManual);
-  XSync (xdisplay, FALSE);
+  if (meta_get_replace_current_wm ())
+    max_retries = 5;
+  else
+    max_retries = 1;
 
-  if (meta_error_trap_pop_with_return (display))
+  n_retries = 0;
+
+  /* Some compositors (like old versions of Mutter) might not properly unredirect
+   * subwindows before destroying the WM selection window; so we wait a while
+   * for such a compositor to exit before giving up.
+   */
+  while (TRUE)
     {
-      g_warning ("Another compositing manager is running on screen %i",
-                 screen_number);
-      return;
+      meta_error_trap_push_with_return (display);
+      XCompositeRedirectSubwindows (xdisplay, xroot, CompositeRedirectManual);
+      XSync (xdisplay, FALSE);
+
+      if (!meta_error_trap_pop_with_return (display))
+        break;
+
+      if (n_retries == max_retries)
+        {
+          /* This probably means that a non-WM compositor like xcompmgr is running;
+           * we have no way to get it to exit */
+          meta_fatal (_("Another compositing manager is already running on screen %i on display \"%s\"."),
+                      screen_number, display->name);
+        }
+
+      n_retries++;
+      g_usleep (G_USEC_PER_SEC);
     }
 
   info = g_new0 (MetaCompScreen, 1);
@@ -550,18 +574,7 @@ meta_compositor_manage_screen (MetaCompositor *compositor,
 
   info->plugin_mgr =
     meta_plugin_manager_get (screen);
-
-  if (info->plugin_mgr != meta_plugin_manager_get_default ())
-    {
-      /* The default plugin manager has been initialized during
-       * global preferences load.
-       */
-      if (!meta_plugin_manager_load (info->plugin_mgr))
-        g_critical ("failed to load plugins");
-    }
-
-  if (!meta_plugin_manager_initialize (info->plugin_mgr))
-    g_critical ("failed to initialize plugins");
+  meta_plugin_manager_initialize (info->plugin_mgr);
 
   /*
    * Delay the creation of the overlay window as long as we can, to avoid
@@ -598,6 +611,14 @@ void
 meta_compositor_unmanage_screen (MetaCompositor *compositor,
                                  MetaScreen     *screen)
 {
+  MetaDisplay    *display       = meta_screen_get_display (screen);
+  Display        *xdisplay      = meta_display_get_xdisplay (display);
+  Window          xroot         = meta_screen_get_xroot (screen);
+
+  /* This is the most important part of cleanup - we have to do this
+   * before giving up the window manager selection or the next
+   * window manager won't be able to redirect subwindows */
+  XCompositeUnredirectSubwindows (xdisplay, xroot, CompositeRedirectManual);
 }
 
 void
@@ -1198,3 +1219,47 @@ meta_get_overlay_window (MetaScreen *screen)
 
   return info->output;
 }
+
+#define FLASH_TIME_MS 50
+
+static void
+flash_out_completed (ClutterAnimation *animation,
+                     ClutterActor     *flash)
+{
+  clutter_actor_destroy (flash);
+}
+
+static void
+flash_in_completed (ClutterAnimation *animation,
+                    ClutterActor     *flash)
+{
+  clutter_actor_animate (flash, CLUTTER_EASE_IN_QUAD,
+                         FLASH_TIME_MS,
+                         "opacity", 0,
+                         "signal-after::completed", flash_out_completed, flash,
+                         NULL);
+}
+
+void
+meta_compositor_flash_screen (MetaCompositor *compositor,
+                              MetaScreen     *screen)
+{
+  ClutterActor *stage;
+  ClutterActor *flash;
+  ClutterColor black = { 0, 0, 0, 255 };
+  gfloat width, height;
+
+  stage = meta_get_stage_for_screen (screen);
+  clutter_actor_get_size (stage, &width, &height);
+
+  flash = clutter_rectangle_new_with_color (&black);
+  clutter_actor_set_size (flash, width, height);
+  clutter_actor_set_opacity (flash, 0);
+  clutter_container_add_actor (CLUTTER_CONTAINER (stage), flash);
+
+  clutter_actor_animate (flash, CLUTTER_EASE_OUT_QUAD,
+                         FLASH_TIME_MS,
+                         "opacity", 192,
+                         "signal-after::completed", flash_in_completed, flash,
+                         NULL);
+}
