@@ -22,9 +22,10 @@
  */
 
 #include <config.h>
-#include "util.h"
-#include "theme.h"
-#include "preview-widget.h"
+#include <meta/util.h>
+#include <meta/theme.h>
+#include "theme-private.h"
+#include <meta/preview-widget.h>
 #include <gtk/gtk.h>
 #include <time.h>
 #include <stdlib.h>
@@ -354,14 +355,15 @@ border_only_contents (void)
   GtkWidget *event_box;
   GtkWidget *vbox;
   GtkWidget *w;
-  GdkColor color;
+  GdkRGBA color;
 
   event_box = gtk_event_box_new ();
 
-  color.red = 40000;
+  color.red = 0.6;
   color.green = 0;
-  color.blue = 40000;
-  gtk_widget_modify_bg (event_box, GTK_STATE_NORMAL, &color);
+  color.blue = 0.6;
+  color.alpha = 1.0;
+  gtk_widget_override_background_color (event_box, 0, &color);
   
   vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 3);
@@ -476,7 +478,7 @@ preview_collection (int font_size,
 {
   GtkWidget *box;
   GtkWidget *sw;
-  GdkColor desktop_color;
+  GdkRGBA desktop_color;
   int i;
   GtkWidget *eventbox;
 
@@ -494,11 +496,12 @@ preview_collection (int font_size,
   
   gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sw), eventbox);
 
-  desktop_color.red = 0x5144;
-  desktop_color.green = 0x75D6;
-  desktop_color.blue = 0xA699;
+  desktop_color.red = 0.32;
+  desktop_color.green = 0.46;
+  desktop_color.blue = 0.65;
+  desktop_color.alpha = 1.0;
 
-  gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &desktop_color);
+  gtk_widget_override_background_color (eventbox, 0, &desktop_color);
 
   i = 0;
   while (i < META_FRAME_TYPE_LAST)
@@ -693,7 +696,7 @@ previews_of_button_layouts (void)
   static gboolean initted = FALSE;
   GtkWidget *box;
   GtkWidget *sw;
-  GdkColor desktop_color;
+  GdkRGBA desktop_color;
   int i;
   GtkWidget *eventbox;
   
@@ -717,11 +720,12 @@ previews_of_button_layouts (void)
   
   gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sw), eventbox);
 
-  desktop_color.red = 0x5144;
-  desktop_color.green = 0x75D6;
-  desktop_color.blue = 0xA699;
+  desktop_color.red = 0.32;
+  desktop_color.green = 0.46;
+  desktop_color.blue = 0.65;
+  desktop_color.alpha = 1.0;
 
-  gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &desktop_color);
+  gtk_widget_override_background_color (eventbox, 0, &desktop_color);
 
   i = 0;
   while (i < BUTTON_LAYOUT_COMBINATIONS)
